import java.net.Socket;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;

/**
 * Ein einfacher Clientantwortprozess fr den Spielserver.
 * 
 * @author Bernd Gramlich 
 * @version 2014-12-25
 */
public class ClientAntwortProzess implements Runnable
{
    private Socket meinSocket;
    private ServerProzess server;
    private String meinNick;
    private Serverprotokoll protokoll;

    private PrintWriter gibAnClient;
    private BufferedReader liesVomClient;

    /**
     * Konstruktor.
     * Erzeuge einen neuen Clientprozess am Socket s fr
     * den Serverprozess p.
     */
    public ClientAntwortProzess (Socket s, ServerProzess p)
    {
        meinSocket = s;
        server     = p;
        meinNick   = null;
        protokoll  = null;
    }

    /**
     * Ablaufplan.
     */
    public void run()
    {
        try
        {
            /*
             * Datenstrme an den und vom Client.
             */
            gibAnClient   = new PrintWriter (meinSocket.getOutputStream (), true);
            liesVomClient = new BufferedReader (new InputStreamReader (meinSocket.getInputStream ()));

            /*
             * Zustndiges Protokoll.
             */
            protokoll = new Serverprotokoll ();

            /*
             * Ein- und Ausgabe und der Serverbefehl.
             */
            String eingabeZeile;
            Antwort ausgabe;
            String befehl;

            /*
             * Aktivitt des Clients.
             */
            boolean aktiv = true;

            /*
             * Reagiere zunchst auf die leere Eingabe vom Client.
             */
            ausgabe = protokoll.verarbeite (null);
            sende (ausgabe.gibNachrichtAnMich ());

            /*
             * Chatte gem Protokoll, solange der Client sendet.
             */
            eingabeZeile = liesVomClient.readLine ();

            if (eingabeZeile == null)
            {
                aktiv = false;
            }

            while (aktiv)
            {
                ausgabe = protokoll.verarbeite (eingabeZeile);
                befehl  = ausgabe.gibBefehl ();

                /*
                 * Wenn eine Nachricht an eine einzelne Adressatin vorliegt,
                 * sende diese.
                 */
                if (null != ausgabe.gibAdressatin () && null != ausgabe.gibNachrichtAnAdressatin ())
                {
                    server.sendeAn (ausgabe.gibAdressatin (), ausgabe.gibNachrichtAnAdressatin (), this);
                }
                /*
                 * Wenn eine Nachricht an alle vorliegt, sende diese.
                 */
                if (null != ausgabe.gibNachrichtAnAlle ())
                {
                    server.sendeAnAlle (ausgabe.gibNachrichtAnAlle ());
                }
                /*
                 * Wenn eine Nachricht an mich selbst vorliegt, sende diese.
                 */
                if (null != ausgabe.gibNachrichtAnMich ())
                {
                    sende (ausgabe.gibNachrichtAnMich ());
                }

                /*
                 * Wenn ein Befehl vorliegt, fhre ihn aus.
                 */
                if (null != befehl)
                {
                    if (befehl.equals ("quit"))
                    {
                        sende (ausgabe.gibNachrichtAnMich ());
                        aktiv = false;
                        break;
                    }
                    else if (befehl.equals ("nick"))
                    {
                        String neuerNick = ausgabe.gibArgument ();
                        boolean nickaenderungErfolgreich = server.aendereNick (meinNick, neuerNick, this);

                        if (nickaenderungErfolgreich)
                        {
                            meinNick = neuerNick;
                            protokoll.chatte ();
                            protokoll.setzeNick (meinNick);
                        }
                    }
                    else if (befehl.equals ("who"))
                    {
                        server.nenneNicks (meinNick, ausgabe.gibArgument ());
                    }
                    else if (befehl.equals ("play"))
                    {
                        server.wuenscheSpiel (meinNick, ausgabe.gibArgument ());
                    }
                    else if (befehl.equals ("move"))
                    {
                        server.spiele (meinNick, ausgabe.gibArgument ());
                    }
                    else if (befehl.equals ("resign"))
                    {
                        server.gibAuf (meinNick);
                    }
                    else if (befehl.equals ("done"))
                    {
                        server.beendeSpiel (meinNick);
                    }
                    else if (befehl.equals ("points"))
                    {
                        server.nennePunkte (meinNick, ausgabe.gibArgument ());
                    }
                }

                eingabeZeile = liesVomClient.readLine ();

                if (eingabeZeile == null)
                {
                    aktiv = false;
                }
            }

            /*
             * Melde Dich vom Server ab.
             */
            server.meldeAb (meinNick);

            /*
             * Schliee die Datenstrme und den Socket.
             */
            gibAnClient.close ();
            liesVomClient.close ();
            meinSocket.close();
        }
        catch (IOException fehler)
        {
            fehler.printStackTrace();
        }
    }

    /**
     * Sende die angegebene Nachricht an den Client.
     */
    public synchronized boolean sende (String nachricht)
    {
        if (gibAnClient != null)
        {
            gibAnClient.println (nachricht);

            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Nenne meine Adresse im Netz.
     */
    public String gibAdresse ()
    {
        return meinSocket.getInetAddress ().getCanonicalHostName () + " ("
        + meinSocket.getInetAddress ().getHostAddress () + ")";
    }

    /**
     * Chatte.
     */
    public void chatte ()
    {
        protokoll.chatte ();
    }
    
    /**
     * Spiele ein Spiel gegen die angegebene Gegnerin.
     */
    public void spiele (String gegnerin)
    {
        protokoll.spiele (gegnerin);
    }

    /**
     * Werte ein beendetes Spiel.
     */
    public void werte ()
    {
        protokoll.werte ();
    }
}